#!/usr/bin/perl -w
# Copyright 2001 Nimrod Zimerman. All Rights Reserved.
# Release 20010630.

use strict;
use Modules::Log;
use Modules::Config;
use Modules::Utils;
use CGI;
use FileHandle;

#
# Accepted parameters:
#  username - A username.
#  password1, password2 - Two supposedly equivalent passwords.
#  name - A real name.
#  country - A country.
#  yearofbirth - A year of birth (4 digits).
#

my $query = new CGI;

my $username    = Utils::defined_or_else ($query->param("username"),    "");
my $password1   = Utils::defined_or_else ($query->param("password1"),   "");
my $password2   = Utils::defined_or_else ($query->param("password2"),   "");
my $name        = Utils::defined_or_else ($query->param("name"),        "");
my $country     = Utils::defined_or_else ($query->param("country"),     "");
my $yearofbirth = Utils::defined_or_else ($query->param("yearofbirth"), "");

validate_input();

check_username_existence ($username);
create_user ($username, $name, $password1);
log_user ($username, $name, $country, $yearofbirth);
emit_user_created_page ($username);

exit;


# Validate the input received from the browser. Terminate on error.
sub validate_input()
{
	my $start_alpha_digits                = '^[a-zA-Z][a-zA-Z0-9]*$';
	my $alpha_digits                      = '^[a-zA-Z0-9]+$';
	my $alpha_digits_spaces_special       = '^[a-zA-Z0-9 \'".()-]+$';
	my $alpha_digits_spaces_special_empty = '^[a-zA-Z0-9 \'".()-]*$';
	my $digits4_integer_empty             = '^(([0-9][0-9][0-9][0-9])|)$';

	my $problems = "";
	
	if (!($username  =~ /$start_alpha_digits/))
	{
		$problems .= "<LI>The username must start with an ".
		             "alpha-betical character, must consist of ".
		             "alpha-numeric characters only and must ".
		             "not be empty.</LI>\n";
	}

	if (!($password1 =~ /$alpha_digits/) ||
	    !($password2 =~ /$alpha_digits/))
	{
		$problems .= "<LI>The passwords must consist of alpha-numeric ".
		             "characters only, and must not be empty.</LI>\n";
	}
	
	if (!($password1 eq $password2))
	{
		$problems .= "<LI>The passwords must be equal.</LI>\n";
	}

	if (!($name =~ /$alpha_digits_spaces_special/))
	{
		$problems .= "<LI>The name supplied contains illegal ".
		             "characters, or is empty.</LI>\n";
	}

	if (!($country =~ /$alpha_digits_spaces_special_empty/))
	{
		$problems .= "<LI>The country supplied contains illegal ".
		             "characters.</LI>\n";
	}

	if (!($yearofbirth =~ /$digits4_integer_empty/))
	{
		$problems .= "<LI>The year of birth must be a 4-digit ".
		             "number.</LI>\n";
	}

	if ($problems ne "")
	{
		print ("Content-Type: text/html\n\n");
		
		my $thtmlFile = FileHandle->new();
		open ($thtmlFile, "<".$Config::incorrect_input_thtml_path) 
		   or die ("Can't open incorrent input thtml file");
		while (<$thtmlFile>)
		{
			s/\|PROBLEMS\|/$problems/g;
			print;
		}
		close ($thtmlFile);
	
		exit;
	}
}

# Check whether the username requested exists on the system. Exit if so.
sub check_username_existence($)
{
	my ($username) = (@_);
	
	if (getpwnam ($username))
	{
		print ("Content-Type: text/html\n\n");
		
		my $thtmlFile = FileHandle->new();
		open ($thtmlFile, "<".$Config::user_exists_thtml_path) 
		   or die ("Can't open user exists thtml file");
		while (<$thtmlFile>)
		{
			s/\|USERNAME\|/$username/g;
			print;
		}
		close ($thtmlFile);
	
		exit;
	}
}

# Create the user requested, using the external useradd wrapper.
sub create_user($$$)
{
	my ($username, $name, $password) = (@_);
	
	system ("./useradd_wrapper", $username, $name, $password) == 0
	   or die ("Problem running useradd");
}

# Log the creation of the user.
sub log_user ($$$$)
{
	my ($username, $name, $country, $yearofbirth) = (@_);

	my ($second, $minute, $hour, $day, $month, $year) =
		localtime (time());
		
	$year += 1900;
	$month++;

	my $logFile = FileHandle->new();
	open ($logFile, ">>".$Config::log_path) 
	   or die ("Can't open log file");
	print $logFile $year.",".$month.",".$day.",".
	               $hour.",".$minute.",".$second.",".
	               $username.",".$name.",".$country.",".$yearofbirth.
	               "\n";
	close ($logFile);
}

# Send a confirmation message to the browser and exit.
sub emit_user_created_page($)
{
	print ("Content-Type: text/html\n\n");
		
	my $thtmlFile = FileHandle->new();
	open ($thtmlFile, "<".$Config::user_created_thtml_path) 
	   or die ("Can't open user created thtml file");
	while (<$thtmlFile>)
	{
		s/\|USERNAME\|/$username/g;
		print;
	}
	close ($thtmlFile);
}	
